﻿var colors = ["blue", "red", "yellow", "green", "black", "orange"];
var gridItemCounter = -1;
var gridColumnCounter = -1;
var gridRowCounter = -1;
var fixedSizingOptions = ["25px", "50px", "75px", "100px", "150px", "200px"];
var flexibleSizingOptions = ["1fr", "2fr", "3fr", "4fr", "5fr", "6fr", "7fr", "8fr", "9fr", "10fr", "11fr", "12fr", "13fr", "14fr", "15fr", "16fr", "17fr", "18fr", "19fr", "20fr"];
var gridSize = { Row: [], Column: [] };

var gridItemTemplate = '\
    <div class="GridItemControl {classes}" id="GridItemControl{targetId}">\
        <div class="ColorSwab" style="background: {color}"></div>\
        <div class="PropertyLabel Property1">grid-column:</div>\
        <div class="PropertyLabel Property2">grid-row:</div>\
        <div class="PropertyLabel Property3">grid-column-span:</div>\
        <div class="PropertyLabel Property4">grid-row-span:</div>\
        <div class="PropertyLabel Property5">grid-layer:</div>\
        <span class="sliderControl Property1" target={targetId} slideroptions="{sliderColumnValues};value=1;readout:left;animateclick=false;setter=updateProperty;" part="grid-column" style="-ms-grid-column: 4; -ms-grid-row: 1;"></span>\
        <span class="sliderControl Property2" target={targetId} slideroptions="{sliderRowValues};value=1;readout:left;animateclick=false;setter=updateProperty;" part="grid-row" style="-ms-grid-column: 4; -ms-grid-row: 2;"></span>\
        <span class="sliderControl Property3" target={targetId} slideroptions="{sliderColumnSpanValues};value=1;readout:left;animateclick=false;setter=updateProperty;" part="grid-column-span" style="-ms-grid-column: 4; -ms-grid-row: 3;"></span>\
        <span class="sliderControl Property4" target={targetId} slideroptions="{sliderRowSpanValues};value=1;readout:left;animateclick=false;setter=updateProperty;" part="grid-row-span" style="-ms-grid-column: 4; -ms-grid-row: 4;"></span>\
        <span class="sliderControl Property5" target={targetId} slideroptions="{sliderRowSpanValues};value=1;readout:left;animateclick=false;setter=updateProperty;" part="grid-layer" style="-ms-grid-column: 4; -ms-grid-row: 5;"></span>\
    </div>\
';

var gridItemMarkupTemplate = '\
        <p class="Indent">&ltdiv id="GridItem{id}" style="</p> \
        <p class="IndentTwice">-ms-grid-column: <span id="grid-columnValue{id}">1</span>;</p>\
        <p class="IndentTwice">-ms-grid-row: <span id="grid-rowValue{id}">1</span>;</p>\
        <p class="IndentTwice">-ms-grid-column-span: <span id="grid-column-spanValue{id}">1</span>;</p>\
        <p class="IndentTwice">-ms-grid-row-span: <span id="grid-row-spanValue{id}">1</span>;</p>\
        <p class="Indent">"&gt&lt/div&gt</p>\
    ';

var gridColumnTemplate = '\
    <div class="Generated PropertyLabel {property}{index}" style="-ms-grid-column: 2; -ms-grid-row: {gridRow};">{property} {displayColumn}</div>\
    <div class="Generated Checkbox {property}{index}" style="-ms-grid-column: 3; -ms-grid-row: {gridRow};"><input id="{property}Checkbox{index}" onchange="toggleSize(\'{property}\',{index});" type="checkbox" /></div>\
    <span class="Generated sliderControl {property}{index}" slideroptions="values=1fr,2fr,3fr,4fr,5fr,6fr,7fr,8fr,9fr,10fr,11fr,12fr,13fr,14fr,15fr,16fr,17fr,18fr,19fr,20fr;value=0;readout:left;animateclick=false;setter=update{property}Size;" sizing="flex" part="{index}" style="-ms-grid-column: 4; -ms-grid-row: {gridRow};"></span>\
';

function initialize(columns, rows) {
    // Initialize values
    gridItemCounter = -1;
    gridColumnCounter = -1;
    gridRowCounter = -1;
    gridSize = { Row: [], Column: [] };

    // create a default grid
    initializeGridStructure(columns, rows);
}

function initializeSample(sample) {
    // Clean out all existing grid items
    while (gridItemCounter > -1)
        removeGridItem();
    //document.querySelector(".

    // Reset the grid and associated controls
    while (gridSize.Column.length > 0) {
        removeRowOrColumn("Column");
    }
    while (gridSize.Row.length > 0) {
        removeRowOrColumn("Row");
    }

    if (sample == "nine") {
        // Simple 9x9 grid, no content
        initialize(9, 9);
    } else if (sample == "column") {
        // Sample 3 column layout with fixed height header & foot and sidebars, variable middle content
        initialize(3, 3);
        setColumnOrRowSize("Column", 0, "fixed", 3);
        setColumnOrRowSize("Column", 2, "fixed", 3);
        setColumnOrRowSize("Row", 0, "fixed", 3);
        setColumnOrRowSize("Row", 2, "fixed", 3);
        document.getElementById("ColumnCheckbox0").checked = true;
        document.getElementById("ColumnCheckbox2").checked = true;
        document.getElementById("RowCheckbox0").checked = true;
        document.getElementById("RowCheckbox2").checked = true;

        // Create the grid items to place
        addNewGridItem(true, 'blue');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'green');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'blue');
        _Slider.initUnder("#GridItemControlContainer");

        var sliders = document.querySelectorAll(".sliderControl[target='0']");
        _Slider.setValue(sliders[2].id, 3);

        var sliders = document.querySelectorAll(".sliderControl[target='1']");
        _Slider.setValue(sliders[1].id, 2);

        var sliders = document.querySelectorAll(".sliderControl[target='2']");
        _Slider.setValue(sliders[0].id, 2);
        _Slider.setValue(sliders[1].id, 2);

        var sliders = document.querySelectorAll(".sliderControl[target='3']");
        _Slider.setValue(sliders[0].id, 3);
        _Slider.setValue(sliders[1].id, 2);

        var sliders = document.querySelectorAll(".sliderControl[target='4']");
        _Slider.setValue(sliders[0].id, 1);
        _Slider.setValue(sliders[1].id, 3);
        _Slider.setValue(sliders[2].id, 3);
    } else if (sample == "logo") {
        initialize(4, 4);
        setColumnOrRowSize("Column", 0, "flex", 19);
        setColumnOrRowSize("Column", 1, "flex", 0);
        setColumnOrRowSize("Column", 2, "flex", 0);
        setColumnOrRowSize("Column", 3, "flex", 19);

        setColumnOrRowSize("Row", 0, "flex", 19);
        setColumnOrRowSize("Row", 1, "flex", 0);
        setColumnOrRowSize("Row", 2, "flex", 0);
        setColumnOrRowSize("Row", 3, "flex", 19);

        addNewGridItem(true, 'red');
        addNewGridItem(true, 'green');
        addNewGridItem(true, 'blue');
        addNewGridItem(true, 'yellow');

        _Slider.initUnder("#GridItemControlContainer");

        var sliders = document.querySelectorAll(".sliderControl[target='1']");
        _Slider.setValue(sliders[0].id, 4);

        var sliders = document.querySelectorAll(".sliderControl[target='2']");
        _Slider.setValue(sliders[1].id, 4);

        var sliders = document.querySelectorAll(".sliderControl[target='3']");
        _Slider.setValue(sliders[0].id, 4);
        _Slider.setValue(sliders[1].id, 4);
    } else if (sample == "complex") {
        initialize(9, 9);
        setColumnOrRowSize("Column", 0, "flex", 1);
        setColumnOrRowSize("Column", 1, "flex", 5);
        setColumnOrRowSize("Column", 2, "flex", 0);
        setColumnOrRowSize("Column", 3, "flex", 5);
        setColumnOrRowSize("Column", 4, "flex", 2);
        setColumnOrRowSize("Column", 5, "flex", 5);
        setColumnOrRowSize("Column", 6, "flex", 0);
        setColumnOrRowSize("Column", 7, "flex", 5);
        setColumnOrRowSize("Column", 8, "flex", 1);

        setColumnOrRowSize("Row", 0, "flex", 2);
        setColumnOrRowSize("Row", 1, "flex", 1);
        setColumnOrRowSize("Row", 2, "flex", 3);
        setColumnOrRowSize("Row", 3, "flex", 1);
        setColumnOrRowSize("Row", 4, "flex", 3);
        setColumnOrRowSize("Row", 5, "flex", 1);
        setColumnOrRowSize("Row", 6, "flex", 3);
        setColumnOrRowSize("Row", 7, "flex", 1);
        setColumnOrRowSize("Row", 8, "flex", 2);

        addNewGridItem(true, 'blue');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'black');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');
        addNewGridItem(true, 'gray');

        _Slider.initUnder("#GridItemControlContainer");

        var sliders = document.querySelectorAll(".sliderControl[target='0']");
        _Slider.setValue(sliders[2].id, 9); // set column span of the header to 9

        var sliders = document.querySelectorAll(".sliderControl[target='1']");
        _Slider.setValue(sliders[0].id, 2);
        _Slider.setValue(sliders[1].id, 3);

        var sliders = document.querySelectorAll(".sliderControl[target='2']");
        _Slider.setValue(sliders[0].id, 2);
        _Slider.setValue(sliders[1].id, 5);

        var sliders = document.querySelectorAll(".sliderControl[target='3']");
        _Slider.setValue(sliders[0].id, 2);
        _Slider.setValue(sliders[1].id, 7);

        var sliders = document.querySelectorAll(".sliderControl[target='4']");
        _Slider.setValue(sliders[0].id, 4);
        _Slider.setValue(sliders[1].id, 3);

        var sliders = document.querySelectorAll(".sliderControl[target='5']");
        _Slider.setValue(sliders[0].id, 4);
        _Slider.setValue(sliders[1].id, 5);

        var sliders = document.querySelectorAll(".sliderControl[target='6']");
        _Slider.setValue(sliders[0].id, 4);
        _Slider.setValue(sliders[1].id, 7);

        var sliders = document.querySelectorAll(".sliderControl[target='7']");
        _Slider.setValue(sliders[0].id, 6);
        _Slider.setValue(sliders[1].id, 3);

        var sliders = document.querySelectorAll(".sliderControl[target='8']");
        _Slider.setValue(sliders[0].id, 6);
        _Slider.setValue(sliders[1].id, 5);

        var sliders = document.querySelectorAll(".sliderControl[target='9']");
        _Slider.setValue(sliders[0].id, 6);
        _Slider.setValue(sliders[1].id, 7);

        var sliders = document.querySelectorAll(".sliderControl[target='10']");
        _Slider.setValue(sliders[0].id, 8);
        _Slider.setValue(sliders[1].id, 3);

        var sliders = document.querySelectorAll(".sliderControl[target='11']");
        _Slider.setValue(sliders[0].id, 8);
        _Slider.setValue(sliders[1].id, 5);

        var sliders = document.querySelectorAll(".sliderControl[target='12']");
        _Slider.setValue(sliders[0].id, 8);
        _Slider.setValue(sliders[1].id, 7);

        var sliders = document.querySelectorAll(".sliderControl[target='13']");
        _Slider.setValue(sliders[0].id, 2);
        _Slider.setValue(sliders[1].id, 4);

        var sliders = document.querySelectorAll(".sliderControl[target='14']");
        _Slider.setValue(sliders[0].id, 2);
        _Slider.setValue(sliders[1].id, 6);

        var sliders = document.querySelectorAll(".sliderControl[target='15']");
        _Slider.setValue(sliders[0].id, 2);
        _Slider.setValue(sliders[1].id, 8);

        var sliders = document.querySelectorAll(".sliderControl[target='16']");
        _Slider.setValue(sliders[0].id, 4);
        _Slider.setValue(sliders[1].id, 4);

        var sliders = document.querySelectorAll(".sliderControl[target='17']");
        _Slider.setValue(sliders[0].id, 4);
        _Slider.setValue(sliders[1].id, 6);

        var sliders = document.querySelectorAll(".sliderControl[target='18']");
        _Slider.setValue(sliders[0].id, 4);
        _Slider.setValue(sliders[1].id, 8);

        var sliders = document.querySelectorAll(".sliderControl[target='19']");
        _Slider.setValue(sliders[0].id, 6);
        _Slider.setValue(sliders[1].id, 4);

        var sliders = document.querySelectorAll(".sliderControl[target='20']");
        _Slider.setValue(sliders[0].id, 6);
        _Slider.setValue(sliders[1].id, 6);

        var sliders = document.querySelectorAll(".sliderControl[target='21']");
        _Slider.setValue(sliders[0].id, 6);
        _Slider.setValue(sliders[1].id, 8);

        var sliders = document.querySelectorAll(".sliderControl[target='22']");
        _Slider.setValue(sliders[0].id, 8);
        _Slider.setValue(sliders[1].id, 4);

        var sliders = document.querySelectorAll(".sliderControl[target='23']");
        _Slider.setValue(sliders[0].id, 8);
        _Slider.setValue(sliders[1].id, 6);

        var sliders = document.querySelectorAll(".sliderControl[target='24']");
        _Slider.setValue(sliders[0].id, 8);
        _Slider.setValue(sliders[1].id, 8);
    }
}

function switchToTab(targetTab) {
    // Disable all tabs
    var tabs = document.querySelectorAll(".Tab");
    for (var i = 0; i < tabs.length; i++) {
        tabs[i].setAttribute("active", false);
    }

    // Disable all tab content
    tabsContents = document.querySelectorAll(".TabContent");
    for (var i=0; i<tabsContents.length; i++) {
        tabsContents[i].setAttribute("active", false);
    }

    // Enable the target tab and content
    activeTabGroup = document.querySelectorAll("." + targetTab);
    for (var i = 0; i < activeTabGroup.length; i++) {
        activeTabGroup[i].setAttribute("active", true);
    }

    // If no markup is visible, hide the markup container 
    var visibleMarkup = document.querySelector(".Markup div[active=true]");
    if (!visibleMarkup) {
        document.querySelector(".Markup").style.display = "none";
    } else {
        document.querySelector(".Markup").style.display = "block";
    }
}

function updateColumnSize(newValue, slider) {
    var column = slider.getAttribute("part");
    gridSize.Column[column] = newValue;

    var newStyle = gridSize.Column.join(" ");
    document.getElementById("ContentGrid").style.msGridColumns = newStyle;

    updateGridMarkup("Columns", newStyle);
}

function updateRowSize(newValue, slider) {
    var row = slider.getAttribute("part");
    gridSize.Row[row] = newValue;

    var newStyle = gridSize.Row.join(" ");
    document.getElementById("ContentGrid").style.msGridRows = newStyle;

    updateGridMarkup("Rows", newStyle);
}

function updateGridMarkup(property, newValue) {
    document.getElementById("Grid"+property+"Value").innerHTML = newValue;
}

function updateGridSize(newValue, slider) {
    var property = slider.getAttribute("part");
    var grid = document.getElementById("ContentGrid");
    grid.style.msTransitionDuration = "1s";
    grid.style.msTransitionProperty = property;
    grid.style[property] = newValue;
}

function updateProperty(newValue, slider) {
    var target = parseInt(slider.getAttribute("target"));
    var part = slider.getAttribute("part");

    document.getElementById("GridItem"+target).style["-ms-" + part] = newValue;

    // Update associated markup
    document.getElementById(part + "Value" + (target+1)).innerHTML = newValue;
}

function setColumnOrRowSize(property, number, sizing, sizeIndex) {
    var slider = document.querySelector(".RowColumnControl[type=" + property + "s] .sliderControl[part='" + number + "']");
    var newSlider = document.createElement("span");
    newSlider.className = slider.className;
    newSlider.setAttribute("part", slider.getAttribute("part"));
    newSlider.style.msGridColumn = slider.style.msGridColumn;
    newSlider.style.msGridRow = slider.style.msGridRow;

    var options = (sizing == "flex") ? flexibleSizingOptions : fixedSizingOptions;

    newSlider.setAttribute("slideroptions", "values=" + options.join(",") + ";value="+sizeIndex+";readout:left;animateClick=false;setter=update" + property + "Size;");
    newSlider.setAttribute("sizing", sizing);
    if (property == "Column") {
        updateColumnSize(options[sizeIndex], newSlider);
    } else {
        updateRowSize(options[sizeIndex], newSlider);
    }
    
    slider.parentNode.insertBefore(newSlider, slider);
    slider.parentNode.removeChild(slider);
    _Slider.initUnder(".RowColumnControl[type=" + property + "s]");
}

function toggleSize(property, number) {
    var slider = document.querySelector(".RowColumnControl[type="+property+"s] .sliderControl[part='" + number + "']");
    if (slider.getAttribute("sizing") == "flex") {
        setColumnOrRowSize(property, number, "fixed", 3);
    } else {
        setColumnOrRowSize(property, number, "flex", 0);
    }
}

function addNewGridItem(deferInitialization, color) {
    createGridItemControls(deferInitialization, gridSize.Column.length, gridSize.Row.length, color);

    updateGridItemMarkup();
}

function removeGridItem() {
    if(gridItemCounter>=0) {
        var controlToRemove = document.getElementById("GridItemControl" + gridItemCounter);
        controlToRemove.parentNode.removeChild(controlToRemove);

        var gridItemToRemove = document.getElementById("GridItem" + gridItemCounter);
        gridItemToRemove.parentNode.removeChild(gridItemToRemove);

        gridItemCounter--;
    }

    updateGridItemMarkup();
}

function updateGridItemMarkup() {
    var gridItemMarkups = document.querySelectorAll("#GridItemMarkup div");
    var gridItemMarkupHolder = document.getElementById("GridItemMarkup");

    if (gridItemMarkups.length > gridItemCounter) {
        // If we've just removed a grid item (note that gridItemCounter is zero-indexed, but gridItemMarkups.length is not), then delete the last markup

        if (gridItemMarkups.length > 0) {
            gridItemMarkupHolder.removeChild(gridItemMarkupHolder.lastChild);
        }
    } else {
        // Otherwise, add a new markup
        var gridItemMarkup = document.createElement("div");
        var markup = gridItemMarkupTemplate.replace(/{id}/g, gridItemCounter + 1);
        gridItemMarkup.innerHTML = markup;
        document.getElementById("GridItemMarkup").appendChild(gridItemMarkup);
    }
}

function addNewRowOrColumn(property) {
    addRowOrColumn(property, gridSize[property].length);
    _Slider.initUnder(".RowColumnControl[type=" + property + "s]");

    var grid = document.getElementById("ContentGrid");

    //Update the grid lines
    if (property == "Column") {
        gridSize.Column.push("1fr");
        var newStyle = gridSize.Column.join(" ");
        grid.style.msGridColumns = newStyle;
        updateGridMarkup("Columns", newStyle);

        // Add the new grid lines
        for (var i = 1; i <= gridSize.Row.length; i++) {
            var gridLines = document.createElement("div");
            gridLines.style.msGridColumn = gridSize[property].length;
            gridLines.style.msGridRow = i;
            gridLines.setAttribute("column", gridSize[property].length);
            gridLines.setAttribute("row", i);
            if (i != gridSize.Row.length) {
                gridLines.style.borderBottom = "1px dotted gray";
            }
            grid.appendChild(gridLines);
        }

        // Update the previous right-most grid lines
        var elems = document.querySelectorAll("#ContentGrid div[column='" + (gridSize[property].length - 1) + "']");
        for (var i = 0; i < elems.length; i++) {
            elems[i].style.borderRight = "1px dotted gray";
        }

        updateColumnSliders();
    } else {
        // If we've added a row we now need to update the borders for each column in that new row
        gridSize.Row.push("1fr");
        var newStyle = gridSize.Row.join(" ");
        grid.style.msGridRows = newStyle;
        updateGridMarkup("Rows", newStyle);

        // Add the new grid lines
        for (var i = 1; i <= gridSize.Column.length; i++) {
            var gridLines = document.createElement("div");
            gridLines.style.msGridColumn = i;
            gridLines.style.msGridRow = gridSize[property].length;
            gridLines.setAttribute("column", i);
            gridLines.setAttribute("row", gridSize[property].length);
            if (i != gridSize.Column.length) {
                gridLines.style.borderRight = "1px dotted gray";
            }

            grid.appendChild(gridLines);
        }

        // Update the previous bottom-most grid lines
        var elems = document.querySelectorAll("#ContentGrid div[row='" + (gridSize[property].length - 1) + "']");
        for (var i = 0; i < elems.length; i++) {
            elems[i].style.borderBottom = "1px dotted gray";
        }

        // Update the row and row-span sliders for each grid item
        updateRowSliders();
    }
}

function updateColumnSliders() {
    // Update the column and column-span sliders for each grid item
    var columnSliders = document.querySelectorAll(".GridItemControl .sliderControl[part='grid-column']");
    var columnSpanSliders = document.querySelectorAll(".GridItemControl .sliderControl[part='grid-column-span']");

    for (var i = 0; i < columnSliders.length; i++) {
        var oldSlider = columnSliders[i];
        var newSlider = document.createElement("span");
        newSlider.className = oldSlider.className;
        newSlider.setAttribute("part", oldSlider.getAttribute("part"));
        newSlider.style.msGridColumn = oldSlider.style.msGridColumn;
        newSlider.style.msGridRow = oldSlider.style.msGridRow;
        newSlider.setAttribute("slideroptions", "min:1;max:" + gridSize.Column.length + ";step:1;value=1;readout:left;animateclick=false;setter=updateProperty;");
        newSlider.setAttribute("target", oldSlider.getAttribute("target"));
        oldSlider.parentNode.insertBefore(newSlider, oldSlider);
        oldSlider.parentNode.removeChild(oldSlider);
    }


    for (var i = 0; i < columnSpanSliders.length; i++) {
        var oldSlider = columnSpanSliders[i];
        var newSlider = document.createElement("span");
        newSlider.className = oldSlider.className;
        newSlider.setAttribute("part", oldSlider.getAttribute("part"));
        newSlider.style.msGridColumn = oldSlider.style.msGridColumn;
        newSlider.style.msGridRow = oldSlider.style.msGridRow;
        newSlider.setAttribute("slideroptions", "min:1;max:" + gridSize.Column.length + ";step:1;value=1;readout:left;animateclick=false;setter=updateProperty;");
        newSlider.setAttribute("target", oldSlider.getAttribute("target"));
        oldSlider.parentNode.insertBefore(newSlider, oldSlider);
        oldSlider.parentNode.removeChild(oldSlider);
    }

    _Slider.initUnder(".GridItemControl");
}

function updateRowSliders() {
    // Update the column and column-span sliders for each grid item
    var rowSliders = document.querySelectorAll(".GridItemControl .sliderControl[part='grid-row']");
    var rowSpanSliders = document.querySelectorAll(".GridItemControl .sliderControl[part='grid-row-span']");

    for (var i = 0; i < rowSliders.length; i++) {
        var oldSlider = rowSliders[i];
        var newSlider = document.createElement("span");
        newSlider.className = oldSlider.className;
        newSlider.setAttribute("part", oldSlider.getAttribute("part"));
        newSlider.style.msGridColumn = oldSlider.style.msGridColumn;
        newSlider.style.msGridRow = oldSlider.style.msGridRow;
        newSlider.setAttribute("slideroptions", "min:1;max:" + gridSize.Row.length + ";step:1;value=1;readout:left;animateclick=false;setter=updateProperty;");
        newSlider.setAttribute("target", oldSlider.getAttribute("target"));
        oldSlider.parentNode.insertBefore(newSlider, oldSlider);
        oldSlider.parentNode.removeChild(oldSlider);
    }


    for (var i = 0; i < rowSpanSliders.length; i++) {
        var oldSlider = rowSpanSliders[i];
        var newSlider = document.createElement("span");
        newSlider.className = oldSlider.className;
        newSlider.setAttribute("part", oldSlider.getAttribute("part"));
        newSlider.style.msGridColumn = oldSlider.style.msGridColumn;
        newSlider.style.msGridRow = oldSlider.style.msGridRow;
        newSlider.setAttribute("slideroptions", "min:1;max:" + gridSize.Row.length + ";step:1;value=1;readout:left;animateclick=false;setter=updateProperty;");
        newSlider.setAttribute("target", oldSlider.getAttribute("target"));
        oldSlider.parentNode.insertBefore(newSlider, oldSlider);
        oldSlider.parentNode.removeChild(oldSlider);
    }

    _Slider.initUnder(".GridItemControl");
}

function removeRowOrColumn(property) {
    var grid = document.getElementById("ContentGrid");

    if (property == "Column") {
        // Find and remove all elements in the last column
        var elems = document.querySelectorAll("#ContentGrid div[column='" + gridSize[property].length + "']");
        for (var i = 0; i < elems.length; i++) {
            elems[i].parentNode.removeChild(elems[i]);
        }

        // Find and update all elements now in the last column
        elems = document.querySelectorAll("#ContentGrid div[column='" + (gridSize[property].length - 1) + "']");
        for (var i = 0; i < elems.length; i++) {
            elems[i].style.borderRightStyle = "none";
        }

        // Remove the controls associated with the removed last column
        elems = document.querySelectorAll("." + property + "" + (gridSize[property].length - 1));
        for (var i = 0; i < elems.length; i++) {
            elems[i].parentNode.removeChild(elems[i]);
        }

        // Update the number of columns and the grid styles
        gridSize.Column.pop();
        var newStyle = gridSize.Column.join(" ");
        grid.style.msGridColumns = newStyle;
        updateGridMarkup("Columns", newStyle);

        updateColumnSliders();
    } else {
        // Find and remove all elements in the last row
        var elems = document.querySelectorAll("#ContentGrid div[row='" + gridSize[property].length + "']");
        for (var i = 0; i < elems.length; i++) {
            elems[i].parentNode.removeChild(elems[i]);
        }

        // find and update all elements now in the last row
        elems = document.querySelectorAll("#ContentGrid div[row='" + (gridSize[property].length - 1) + "']");
        for (var i = 0; i < elems.length; i++) {
            elems[i].style.borderBottomStyle = "none";
        }

        // Remove the controls associated with the removed last row
        elems = document.querySelectorAll("." + property + "" + (gridSize[property].length - 1));
        for (var i = 0; i < elems.length; i++) {
            elems[i].parentNode.removeChild(elems[i]);
        }

        // Update the number of columns and the grid styles
        gridSize.Row.pop();
        var newStyle = gridSize.Row.join(" ");
        grid.style.msGridRows = newStyle;
        updateGridMarkup("Rows", newStyle);

        updateRowSliders();
    }
}

function initializeGridStructure(columns, rows) {
    var grid = document.getElementById("ContentGrid");

    for (var i = 0; i < columns; i++) {
        addRowOrColumn("Column", i);
    }

    for (var i = 0; i < rows; i++) {
        addRowOrColumn("Row", i);
    }


    for (var i = 0; i < columns; i++) {
        gridSize.Column.push("1fr");
    }
    for (var j = 0; j < rows; j++) {
        gridSize.Row.push("1fr");
    }

    var newColumnStyle = gridSize.Column.join(" ");
    var newRowStyle = gridSize.Row.join(" ");
    grid.style.msGridColumns = newColumnStyle;
    grid.style.msGridRows = newRowStyle;
    updateGridMarkup("Columns", newColumnStyle);
    updateGridMarkup("Rows", newRowStyle);

    // Initialize the grid to a predefined size
    initializeGridLines(gridSize.Column.length, gridSize.Row.length);

    _Slider.initUnder(".RowColumnControl");
}

function addRowOrColumn(property, number) {
    var controlMarkup = gridColumnTemplate.replace(/{gridRow}/g, number + 2);
    controlMarkup = controlMarkup.replace(/{displayColumn}/g, number + 1);
    controlMarkup = controlMarkup.replace(/{index}/g, number);
    controlMarkup = controlMarkup.replace(/{property}/g, property);

    var container = document.createElement("div");
    container.innerHTML = controlMarkup;

    var controlContainer = document.querySelector(".RowColumnControl[type=" + property + "s]");

    while (container.hasChildNodes()) {
        controlContainer.appendChild(container.removeChild(container.firstChild));
    }
}

function createGridItemControls(deferInitialization, columns, rows, color) {
    var grid = document.getElementById("ContentGrid");
    var currentControlMarkup = "";
 
    gridItemCounter++;

    var controlMarkup = gridItemTemplate.replace(/{sliderColumnValues}/g, "min:1;max:" + columns + ";step:1");
    controlMarkup = controlMarkup.replace(/{sliderRowValues}/g, "min:1;max:" + rows + ";step:1");
    controlMarkup = controlMarkup.replace(/{sliderColumnSpanValues}/g, "min:1;max:" + columns + ";step:1");
    controlMarkup = controlMarkup.replace(/{sliderRowSpanValues}/g, "min:1;max:" + rows + ";step:1");
    controlMarkup = controlMarkup.replace(/{classes}/g, "");
    controlMarkup = controlMarkup.replace(/{color}/g, (color)? color : colors[gridItemCounter%6]);
    currentControlMarkup += controlMarkup.replace(/{targetId}/g, gridItemCounter);

    var gridItem = document.createElement("div");
    gridItem.id = "GridItem" + gridItemCounter;
    gridItem.className = "GridItem" + gridItemCounter;
    gridItem.style.backgroundColor = (color) ? color : colors[gridItemCounter % 6];
    gridItem.style.opacity = 0.5;
    grid.appendChild(gridItem);

    var newNode = document.createElement("div");
    newNode.innerHTML = currentControlMarkup;

    document.getElementById("GridItemControlContainer").appendChild(newNode);
 
    if (!deferInitialization) {
        _Slider.initUnder("#GridItemControl" + gridItemCounter);
    }
}

// Build a grid
function initializeGridLines(columns, rows) {
    var grid = document.getElementById("ContentGrid");

    // Create the grid lines
    for (var i = 1; i <= columns; i++) {
        for (var j = 1; j <= rows; j++) { 
            var gridLines = document.createElement("div");
            gridLines.style.msGridColumn = i;
            gridLines.style.msGridRow = j;
            gridLines.setAttribute("column",i);
            gridLines.setAttribute("row",j);
            if(i != columns)
                gridLines.style.borderRight = "1px dotted gray";
            if (j != rows)
                gridLines.style.borderBottom = "1px dotted gray";
            grid.appendChild(gridLines);
        }
    }
}


//  turn off background if asked
if (location.search.match(/\bbg=none\b/i))
    document.body.parentNode.className = "nobackground";

