﻿
function ColorPickerRGB(id, onUpdate, initialColorString) {
    var canvas = document.getElementById(id);
    var ctx = canvas.getContext("2d");

    this.setColorString = function (cssString) {
        SetColorFromString(cssString);
    };
    
    //  layout values
    var canvasHeight = canvas.height;
    var canvasWidth = canvas.width;
    var margin = 8;
    var barHeight = Math.floor((canvasHeight - margin * 8) / 4);
    var barWidth = canvasWidth - margin * 2;

    //  default values
    var colorComponents = [];
    colorComponents[0] = 0.5;
    colorComponents[1] = 0.5;
    colorComponents[2] = 0.5;
    colorComponents[3] = 1;

    function CreateColorArray(bitIndex) {
        var array = ctx.createImageData(barWidth, barHeight);
        var bits = array.data;

        var index = 0;
        for (var y = 0; y < array.height; ++y) {
            for (var x = 0; x < array.width; ++x) {
                bits[index] = colorComponents[0] * 255;
                bits[index + 1] = colorComponents[1] * 255;
                bits[index + 2] = colorComponents[2] * 255;
                bits[index + 3] = 255;
                bits[index + bitIndex] = (array.width - x) / (array.width - 1) * 255;
                index += 4;
            }
        }

        return array;
    }

    function CreateAlphaArray() {
        var array = ctx.createImageData(barWidth, barHeight);
        var bits = array.data;

        var index = 0;
        var grayValues = [0.75, 1, 0.75, 1];
        for (var y = 0; y < array.height; ++y) {
            var yCheck = Math.floor((y % 12) / 6);
            for (var x = 0; x < array.width; ++x) {
                var xCheck = Math.floor((x % 12) / 6);
                var xAlpha = (array.width - x) / (array.width - 1);
                var grayValue = grayValues[yCheck + xCheck] * (1 - xAlpha);
                bits[index++] = (colorComponents[0] * xAlpha + grayValue) * 255;
                bits[index++] = (colorComponents[1] * xAlpha + grayValue) * 255;
                bits[index++] = (colorComponents[2] * xAlpha + grayValue) * 255;
                bits[index++] = 255;
            }
        }

        return array;
    }

    function DrawColorBar(colorIndex, value) {
        var yTop = margin + (barHeight + margin * 2) * colorIndex;
        var yBottom = yTop + barHeight;
        var xIndicator = margin + 0.5 + Math.round((1 - value) * (barWidth - 1));
        ctx.clearRect(0, yTop - margin, margin + barWidth + margin, margin + barHeight + margin);
        ctx.putImageData(colorArrays[colorIndex], margin, yTop);

        ctx.beginPath();

        ctx.moveTo(xIndicator, yTop);
        ctx.lineTo(xIndicator - 5, yTop - 5);
        ctx.lineTo(xIndicator + 5, yTop - 5);
        ctx.lineTo(xIndicator, yTop);

        ctx.moveTo(xIndicator, yBottom);
        ctx.lineTo(xIndicator - 5, yBottom + 5);
        ctx.lineTo(xIndicator + 5, yBottom + 5);
        ctx.lineTo(xIndicator, yBottom);

        ctx.fillStyle = "#555";
        ctx.fill();
    }

    // conversion functions directly from http://www.w3.org/TR/css3-color/#hsl-color
    function hsl2rgb(h, s, l) {
        var m2 = (l <= 0.5) ? l * (s + 1) : l + s - l * s;
        var m1 = l * 2 - m2;
        var r = hue2rgb(m1, m2, h + 1 / 3);
        var g = hue2rgb(m1, m2, h);
        var b = hue2rgb(m1, m2, h - 1 / 3);
        return { r: r, g: g, b: b };
    }

    function hue2rgb(m1, m2, h) {
        if (h < 0) h = h + 1;
        if (h > 1) h = h - 1;
        if (h * 6 < 1) return m1 + (m2 - m1) * h * 6;
        if (h * 2 < 1) return m2;
        if (h * 3 < 2) return m1 + (m2 - m1) * (2 / 3 - h) * 6;
        return m1;
    }

    function SetColorFromString(cssString) {
        var rgb = { r: 0, g: 0, b: 0 }, a = 1;

        var hashMatch = cssString.match(/#([0-9a-f]{2})([0-9a-f]{2})([0-9a-f]{2})/i);
        if (hashMatch) {
            rgb = { r: parseInt(hashMatch[1], 16) / 255, g: parseInt(hashMatch[2], 16) / 255, b: parseInt(hashMatch[3], 16) / 255 };
        }

        var rgbaMatch = cssString.match(/rgba?\(\s*([\d\.]+),\s*([\d\.]+),\s*([\d\.]+)(?:,\s*([\d\.]+))?\s*\)/);
        if (rgbaMatch) {
            rgb = { r: parseFloat(rgbaMatch[1]) / 255, g: parseFloat(rgbaMatch[2]) / 255, b: parseFloat(rgbaMatch[3]) / 255 };
            if (rgbaMatch[4])
                a = parseFloat(rgbaMatch[4]);
        }

        var hslaMatch = cssString.match(/hsla?\(\s*([\d\.]+),\s*([\d\.]+)%,\s*([\d\.]+)%(?:,\s*([\d\.]+))?\s*\)/);
        if (hslaMatch) {
            rgb = hsl2rgb(parseFloat(hslaMatch[1]) / 360, parseFloat(hslaMatch[2]) / 100, parseFloat(hslaMatch[3]) / 100 );
            if (hslaMatch[4])
                a = parseFloat(hslaMatch[4]);
        }

        if (cssString.match(/white/i))
            rgb = { r: 1, g: 1, b: 1 };
        else if (cssString.match(/transparent/i))
            a = 0;

        colorComponents[0] = rgb.r;
        colorComponents[1] = rgb.g;
        colorComponents[2] = rgb.b;
        colorComponents[3] = a;

        RedrawAll();
    }

    function normalizedColorToHexString() {
        var ret = "#";
        for (var i = 0; i < arguments.length; i++) {
            var arg = arguments[i];
            ret += (arg < 16 ? "0" : "") + arg.toString(16);
        }
        return ret.toUpperCase();
    }

    function UpdateReadout() {
        var r = Math.round(colorComponents[0] * 255);
        var g = Math.round(colorComponents[1] * 255);
        var b = Math.round(colorComponents[2] * 255);
        var a = Math.round(colorComponents[3] * 100) / 100;

        var colorString;

        if (a != 1)
            colorString = "rgba(" + r.toString() + ", " + g.toString() + ", " + b.toString() + ", " + a.toString() + ")";
        else if (r == 255 && g == 255 && b == 255)
            colorString = "white";
        else if (r == 0 && g == 0 && b == 0)
            colorString = "black";
        else
            colorString = normalizedColorToHexString(r, g, b);

        if (onUpdate)
            onUpdate(colorString);
    }

    var colorArrays = [];
    var movingColor = -1;

    function RedrawAll() {
        for (var i = 0; i < 4; ++i) {
            if (i != movingColor)
                colorArrays[i] = i < 3 ? CreateColorArray(i) : CreateAlphaArray();

            DrawColorBar(i, colorComponents[i]);
        }

        UpdateReadout();
    }

    function pointerStart(target, pointerId, x, y) {
        movingColor = Math.floor(y / (barHeight + margin * 2));

        if (movingColor > 3) {
            movingColor = -1;
        }
        else {
            pointerMove(target, pointerId, x, y);
        }
    }

    function pointerEnd(target, pointerId) {
            movingColor = -1;
    }

    function pointerMove(target, pointerId, x, y) {
        if (movingColor != -1) {
            colorComponents[movingColor] = Math.max(0, Math.min(1, 1 - (x - margin) / (barWidth - 1)));
            RedrawAll();
        }
    }

    PointerDraw(canvas, pointerStart, pointerMove, pointerEnd);

    if (initialColorString)
        SetColorFromString(initialColorString);
    else
        RedrawAll();
}

